from .common import CommonStrings


class ArgParserStrings:
    """Strings for the physbiblio.argParser module"""

    citHelp = (
        "update citation counts for all entries using info from the INSPIRE database"
    )
    cleanHelp = "clean the entries in the database"
    cleanStartHelp = "the index from which the cleaning should start"
    cliHelp = "open the internal command line interface"
    closeMainW = "Closing main window..."
    dailyHelp = "fetch the daily updates from INSPIRE-HEP OAI"
    datesEndHelp = "the ending date (format as yyyy-mm-dd)"
    datesHelp = "fetch the updates from INSPIRE-HEP OAI between the given dates"
    datesStartHelp = "the starting date (format as yyyy-mm-dd)"
    exportFilenameHelp = "the filename where to save the entries"
    exportHelp = "export all the entries in the database in a file"
    guiHelp = "open the gui"
    profileHelp = "define the profile that must be used"
    subHelp = "sub-command help"
    testFailed = "Some error occurred during tests"
    testHelp = "run the test suite"
    testDbHelp = "do not perform database tests"
    testGuiHelp = "do not perform gui tests"
    testLongHelp = "do not perform long tests"
    testOAIHelp = "do not perform online tests with INSPIRE OAI"
    testOnlineHelp = "do not perform online tests"
    texHelp = "read .tex file(s) and create a *.bib file with the cited bibtexs"
    texBibHelp = "the filename of the bib file where to write"
    texOverwriteHelp = "overwrite the bib file, if existing"
    texRemoveHelp = (
        "remove from the .bib file those elements "
        + "that are not more used in the tex"
    )
    texReorderHelp = "reorder the elements in the .bib file and add new ones"
    texSaveHelp = "save the changes in the database"
    texTexsHelp = (
        "the filename of the tex file to read. A folder or wildcards are admitted"
    )
    texUpdateHelp = (
        "update the elements in the .bib file if they changed "
        + "in the database and remove duplicated entries"
    )
    updateHelp = "use INSPIRE to update the information in the database"
    updateForceHelp = "force the update"
    updateStartHelp = "the index from which the updating should start"
    weeklyHelp = "fetch the weekly updates from INSPIRE-HEP OAI"


class BibtexWriterStrings:
    """Strings for the physbiblio.bibtexWriter module"""

    errorNotString = "The field %s in entry %s must be a string"


class CLIStrings:
    """Strings for the physbiblio.cli module"""

    activate = (
        "[CLI] Activating CommandLineInterface\n"
        + "Write a command and press Enter ('Ctrl+D' or 'exit()' to exit)."
    )
    close = "CommandLineInterface closed."


class ConfigStrings:
    """Strings for the physbiblio.config module"""

    class Desc:
        """Description of some parameters"""

        ADSToken = "Token for connecting to the ADS service by NASA"
        autoResize = "Automatically resize columns and rows in the main bibtex table"
        batchSizeInspire = (
            "Batch size for cumulative searches through the INSPIRE API "
            + "(it must be <= 1000)"
        )
        bibListCols = "The columns to be shown in the entries list"
        confirmExit = "Confirm before exiting"
        defaultCat = "Default categories for imported bibtexs"
        fetchAbstract = (
            "Automatically fetch the abstract from arXiv if an arxiv number is present"
        )
        fontSize = "Font size in the list of bibtex entries and companion boxes"
        limitBibtexs = "Number of bibtex entries in the initial view of the main table"
        logFName = "Name of the log file"
        logLevel = (
            "How many messages to save in the log file "
            + "(will have effects only after closing the application)"
        )
        mainDBName = "Name of the database file"
        maxAPIRes = "Max number of entries per page when reading external API results"
        maxAuthorsD = "Max number of authors to be displayed in the main list"
        maxAuthorsS = "Max number of authors to be saved when adding info from arXiv"
        maxSavedSearches = "Max number of automatically saved search/replace arguments"
        notifyUpdate = (
            "If configured to False, do not show the existence"
            + " of updates when opening the app"
        )
        PDFApp = "Application for opening PDF files (used only via command line)"
        PDFFolder = "Folder where to save the PDF files"
        sinceLastUpdate = (
            "Parameter that saves the number of the last used version"
            + " for showing the list of changes when a new one is opened"
        )
        timeout = "Timeout for the web queries"
        updateFrom = (
            "Index of bibtex entries (firstdate ASC) "
            + 'from which I should start when using "Update bibtexs"'
        )
        webApp = "Web browser (used only via command line)"

    confEntryInsert = "No settings found with this name (%s). Inserting it."
    confEntryUpdate = "An entry with the same name is already present. Updating it"
    confLoaded = "Configuration loaded.\n"
    defaultCfgPath = "Default configuration path: %s"
    defaultDataPath = "Default data path: %s"
    defaultProfileChanged = "Default profile changed to %s"

    errorDeleteProfile = "Cannot delete profile"
    errorInsertProfile = "Cannot insert profile"
    errorInvalidFieldI = "Invalid field or identifierField: %s, %s"
    errorListProfilesMatch = "List of profile names does not match existing profiles!"
    errorName = "No name given!"
    errorNameFilenameOne = (
        "You should specify the name or the filename associated with the profile"
    )
    errorNameFilenameOnly = (
        "You should specify only the name "
        + "or only the filename associated with the profile"
    )
    errorNoProfileUseDef = (
        "The profile '%s' does not exist! Back to the default one ('%s')"
    )
    errorNoProfilesName = "No profiles with the given name!"
    errorOrder = "No order given!"
    errorProfileDefault = (
        "Something went wrong when setting new default profile. Undoing..."
    )
    errorProfileName = "You must provide the profile name!"
    errorProfileNotFound = "Profile not found!"
    errorProfileOrdering = (
        "Something went wrong when setting new profile order. Undoing..."
    )
    errorReadConf = "ERROR: reading config from '%s' failed."
    errorReadParam = "Failed in reading parameter '%s'."
    errorSearchField = (
        "Empty value or field not in the following list: "
        + "[searchDict, replaceFields, name, limitNum, offsetNum].\n"
        + "Field: %s, value: %s"
    )
    errorUpdateProfile = "Cannot update profile"
    invalidType = "Invalid parameter type: %s"
    noAttribute = "Attribute does not exist for current ConfigParameter: %s"
    readConf = "Reading configuration.\n"
    restartWProfile = "Restarting with profile '%s', database: %s"
    startWProfile = "Starting with profile '%s', database: %s"


class DatabaseCoreStrings:
    """Strings for the physbiblio.databaseCore module"""

    closeDb = "Closing database..."
    errorAlterEntries = "Cannot alter table 'entries'!"
    errorCannotCommit = "Impossible to commit!"
    errorCannotRollback = "Impossible to rollback!"
    errorConnection = "Connection error: %s\nquery: %s"
    errorCreateTable = "Create table %s failed"
    errorCursor = 'Cursor error: %s\nThe query was: "%s"\nand the parameters: %s'
    errorInsMainCats = "Insert main categories failed"
    errorInsUpd = "Cannot insert/update: ID exists!\n%s\nquery: %s"
    errorLiteralEval = "Error in literal_eval with string '%s'"
    invalidIsLocked = "Invalid `self.onIsLocked`!"
    newColEntries = "New column in table 'entries': '%s' (%s)."
    noDatabaseCreate = "-------New database or missing tables.\nCreating them!\n\n"
    openDb = "Opening database: %s"
    opErDbOpen = (
        "OperationalError: the database is already open "
        + "in another instance of the application\n"
        + "query failed: %s"
    )
    rollbackDb = "Rolled back to last commit."
    savedDb = "Database saved."


class DatabaseStrings(CommonStrings, DatabaseCoreStrings):
    """Strings for the physbiblio.database module"""

    catNotInDb = "Category '%s' not in database"
    cleaning = "Cleaning (%s, %s)"
    errorReadInput = "Something failed in reading your input '%s'"
    errorRename = "Cannot rename folder"
    newDbCreate = "-------New database. Creating tables!\n\n"
    noElsFound = "No elements found?"

    class Bibs:
        """Strings for the physbiblio.database. class"""

        alreadyExisting = "Already existing: %s\n"
        apiChanged = "%d changed entries:\n%s"
        apiDone = "Inspire OAI harvesting done!"
        apiInfoS = "-- %s, '%s' -> '%s'"
        apiInfoL = "-- %s"
        apiInfoP = "Processing: %s"
        apiNew = "new:\n%s"
        apiOld = "old:\n%s"
        book = "Book"
        callOAIDates = "Calling INSPIRE-HEP OAI harvester between dates %s and %s"
        cannotParse = "Cannot parse properly:\n%s"
        cbError = "Error while cleaning entry '%s'"
        cbInvalidStart = "Invalid startFrom in cleanBibtexs"
        cbProcessProgr = "%5d / %d (%5.2f%%) - cleaning: '%s'\n"
        cbProcessTot = "CleanBibtexs will process %d total entries"
        cbResChan = "%d bibtex entries changed"
        cbResEntr = "%d entries processed"
        cbResErr = "%d errors occurred"
        ccCannotReadId = (
            "Cannot read the INSPIRE ID from the recently retrieved record %s"
        )
        ccCannotUpdateFields = (
            "Cannot update citation fields for entry %s from record %s"
        )
        ccNoCorrespondence = "No correspondence in database for INSPIRE ID %s"
        ccProcessTot = "CitationCount will process %d total entries"
        ccProcessProgr = "%5d / %d (%5.2f%%) - citation count: '%s'"
        delete = "Delete entry, using key = '%s'"
        elementChanged = "-- element changed!"
        elementsFound = "%d elements found"
        emptyBibtex = "Empty bibtex?\n%s\n%s"
        errorField = "Non-existing field or unappropriated value: (%s, %s, %s)"
        errorGFNoElement = "Error in getField('%s', '%s'): no element found?"
        errorGFNoField = "Error in getField('%s', '%s'): the field is missing?"
        errorOAIEntryDet = "something wrong with entry %s\n%s"
        errorOAIEntryMisKey = "Key %s missing for entry:\n%s"
        errorOAIEntryG = "Something went wrong with this entry..."
        errorParseBibtex = "Problem in parsing the following bibtex code:\n%s"
        errorParseBib = "Impossible to parse bibtex!"
        errorReadBibdict = "Cannot read bibdict:\n'%s'"
        errorQueryFailed = "Query failed: %s\nvalues: %s"
        errorUIIDArxiv = "Something went wrong in updateInspireID with arxiv search"
        errorUIIDDOI = "Something went wrong in updateInspireID with doi search"
        errorUIIDGeneric = "Something went wrong in updateInspireID"
        errorUpdateBib = "Impossible to update bibkey"
        experimental = "Experimental paper"
        fcbBadEntries = "%d bad entries found:\n %s"
        fcbInvalidStart = "Invalid startFrom in findCorruptedBibtexs"
        fcbNotReadable = "%s is NOT readable!\n"
        fcbProcessProgr = "%5d / %d (%5.2f%%) - processing: '%s'"
        fcbProcessTot = "findCorruptedBibtexs will process %d total entries"
        fcbReadable = "%s is readable.\n"
        gffaDone = "\n\ngetFieldsFromArxiv has finished!"
        gffaFailed = "Cannot get and save info from arXiv!\n"
        gffaProcessProgr = "%5d / %d (%5.2f%%) - processing: arxiv:%s\n"
        gffaProcessTot = "getFieldsFromArxiv will process %d entries."
        gffaSummary = "%d entries processed, of which these %d generated errors:\n%s"
        ifbCompleteSummary = (
            "Import completed.\n"
            + "%d entries processed, of which %d existing, "
            + "%d successfully inserted and %d errors."
        )
        ifbEmptyKey = "Impossible to insert an entry with empty bibkey!\n"
        ifbExist = "Already existing: %s\n"
        ifbFailed = "Failed in inserting entry %s\n"
        ifbFromFile = "Importing from file bib: %s"
        ifbInserted = "Element successfully inserted.\n"
        ifbNewKey = "Entry will have key: '%s'"
        ifbProcessProgr = "%5d / %d (%5.2f%%), processing entry %s"
        ifbProcessTot = "Entries to be processed: %d"
        iidEmptyID = "InspireID is empty, cannot proceed."
        iidEmptyRecord = "Empty record looking for recid:%s!"
        iidKeyError = "Key error: (%s, %s)"
        iidSaved = "Inspire OAI info for %s saved."
        iidStMissing = "Something missing in entry %s"
        iidWrongType = "Wrong type in inspireID: %s"
        iidWrongVal = "Wrong value/format in inspireID: %s"
        invalidComparisonOp = (
            "Invalid comparison operator "
            + "('%s') in database operations!\n"
            + "Reverting to default '='."
        )
        invalidLogicalOp = (
            "Invalid logical connection operator "
            + "('%s') in database operations!\n"
            + "Reverting to default 'and'."
        )
        invalidOrdering = (
            "Invalid ordering "
            + "('%s') in database operations!\n"
            + "Reverting to default 'ASC'."
        )
        laiEmptyKey = "Impossible to insert an entry with empty bibkey!\n%s\n"
        laiErrors = "ERRORS!\nFailed to load and import entries:\n%s"
        laiFailed = "Failed in inserting entry %s\n"
        laiImported = "Imported entries:\n%s"
        laiInserted = "Element successfully inserted.\n"
        laiInvalidArgs = "Invalid arguments!"
        laiInvalidMethod = "Method not valid: %s"
        laiMismatch = (
            "Possible mismatch. Specify the number of element "
            + "to select with 'number'\n%s"
        )
        laiNewKey = "Entry will have key: '%s'"
        laiProcessProgr = "%5d / %d (%5.2f%%) - looking for string: '%s'\n"
        laiProcessTot = "LoadAndInsert will process %d total entries"
        laiReadError = "Error while reading the bibtex '%s'"
        lecture = "Lecture"
        noneType = "No type"
        pabFound = "%d bibtex entries found."
        pabProgr = "%4d - %s\n"
        pabRead = "Reading bibtex entry #%d"
        pabStart = "Start reading file content"
        parsingExcPU = "Parsing exception in prepareUpdate:\n%s\n%s"
        phdth = "PhD thesis"
        proceeding = "Proceeding"
        psbEmpty = "Impossible to parse empty text!"
        psbError = "Impossible to parse text:\n%s"
        psbProcess = "Processing:\n%s"
        replaceInvalidNew = "Invalid 'fiNews' or 'news' (they must be lists)"
        replaceMissingField = "Field %s not found in entry %s"
        replaceProcessProgr = "processing %5d / %d (%5.2f%%): entry %s"
        replaceProcessTot = "Replace will process %d entries"
        replaceText = "Replacing text in entries: "
        replaceError = "Something wrong in replace"
        review = "Review"
        souError = (
            "Something wrong here. "
            + "Possibly the bibtex key has been changed "
            + "while processing entry '%s'?"
        )
        souFirst = "Starting preliminary loop"
        souInvalidStart = "Invalid startFrom in searchOAIUpdates"
        souResChan = "%d entries changed"
        souResErr = "%d errors occurred"
        souResProc = "%d entries processed"
        souProcessProgr = "%5d / %d (%5.2f%%) - looking for update: '%s'"
        souProcessTot = "SearchOAIUpdates will process %d total entries"
        souSecond = "Starting main loop"
        updateBibkey = "Updating bibkey for entry '%s' into '%s'"
        updateField = "Updating '%s' for entry '%s'"
        useQueryVals = "Using query:\n%s\nvalues: %s"
        wrongTypeField = "Wrong type or value for field %s (%s)?"

        class Search:
            """Stuff used in searches"""

            invalidContent = "Invalid 'content' in search: '%s' (%s)"
            invalidField = "Invalid field: '%s'"
            invalidIds = "Invalid list of ids: '%s'"
            invalidOperator = "Invalid operator: '%s'"
            opCEAll = "all the following"
            opCENone = "none of the following"
            opCEOne = "at least one among"
            opCSub = "this or subcategories"
            opTContains = "contains"
            opTDifferent = "different from"
            opTExact = "exact match"
            opTNotCont = "does not contain"

    class BibsCats:
        """Strings for the physbiblio.database.CatsEntries class"""

        askInputBib = "entries for '%d': "
        askInputCat = "categories for '%s': "
        alreadyPresent = "EntryCat already present: (%d, %s)"
        insert = "inserting (idCat=%s and key=%s)"
        updateKey = "Updating entryCats for bibkey change, from '%s' to '%s'"

    class BibsExps:
        """Strings for the physbiblio.database.EntryExps class"""

        askInputBib = "entries for '%d': "
        askInputExp = "experiments for '%s': "
        alreadyPresent = "EntryExp already present: (%s, %d)"
        insert = "inserting (key=%s and idExp=%s)"
        updateKey = "Updating entryExps for bibkey change, from '%s' to '%s'"

    class Cats:
        """Strings for the physbiblio.database.Categories class"""

        alreadyPresent = (
            "An entry with the same name is already present in the same category!"
        )
        cannotDelete = "You should not delete the category with id: %d%s."
        errorExtract = "Error in extracting category by idCat"
        invalidDepth = "Invalid depth in printCatHier (must be greater than 2)"
        lookChild = "Looking for child categories"
        missingField = "Missing field when inserting category"
        updateField = "Updating '%s' for category '%s'"
        useCat = "Using idCat=%d"

    class CatsExps:
        """Strings for the physbiblio.database.CatsExps class"""

        askInputCat = "categories for '%s': "
        askInputExp = "experiments for '%d': "
        alreadyPresent = "ExpCat already present: (%d, %d)"
        insert = "inserting (idCat=%s and idExp=%s)"

    class Exps:
        """Strings for the physbiblio.database.Experiments class"""

        errorExtract = "Error in extracting experiment by idExp"
        errorPrint = "Error printing experiments!"
        updateField = "Updating '%s' for experiment '%s'"
        useExp = "Using idExp=%d"


class ErrorsStrings:
    """Strings for the physbiblio.errors module"""

    errorAddHandler = "Failed while trying to add a new handler"
    unhandled = "Unhandled exception"


class ExportStrings:
    """Strings for the physbiblio.export module"""

    autoSave = "Changes will be automatically saved at the end!"
    byPhysbiblio = "%file written by PhysBiblio\n"
    cannotCreate = "Cannot create file %s!"
    cannotRead = "Cannot read file %s.\nCreating one."
    cannotRemoveBackup = "Cannot remove backup file.\nCheck the file permissions."
    cannotRestoreBackup = "Cannot restore backup file.\nCheck the file permissions."
    cannotWriteBackup = "Cannot write backup file.\nCheck the folder permissions."
    cannotWrite = "Cannot write on file.\nCheck the file permissions."
    citeFound = r"%d \cite commands found in .tex file"
    doneAllTexs = "Done for all the texFiles.\n\n"
    errorCitation = "Cannot recognize citation list in '%s'!"
    errorExport = "Problems in exporting .bib file!"
    errorLoading = "Problems in loading the .bib file!"
    errorNoFile = "The file %s does not exist."
    errorWrite = "Impossible to write file '%s'"
    entriesNotFound = "Impossible to find %d entries:"
    entriesRemoved = "Output file updated. Removed entries:\n%s"
    entryInserted = "%s inserted in output file"
    keyMissing = "Key '%s' missing, trying to import it from Web"
    keysFound = "%d keys found in .tex file"
    missingEntries = "%d required entries were missing in database"
    newKeysFound = "%d new keys found in .tex file"
    newKeysTotal = "%d new keys found, %d in total."
    noElement = "No elements to export!"
    noLastSel = "No last selection to export!"
    nonMatchingEntries = "Possible non-matching keys in %d entries:\n"
    outputUpdated = "Output file updated"
    readFrom = "Reading keys from '%s'"
    resume = "\n\nRESUME"
    retrievedEntries = "%d new entries retrieved:"
    saveTo = "Saving in '%s'"
    startEFTF = "Starting exportForTexFile...\n\n"
    unexpectedEntry = "Unexpected error in extracting entry '%s' to the output file"
    unexpectedForEntries = "Unexpected errors for %d entries:"
    totalWarnings = "     %s warning(s) occurred!"


class InspireStatsStrings(CommonStrings):
    """Strings for the physbiblio.inspireStats module"""

    authorStats = "Stats for author '%s'"
    authorStatsCompleted = "Stats for author '%s' completed!"
    authorStatsLooking = "%5d / %d (%5.2f%%) - looking for paper: '%s'"
    authorStatsProcess = (
        "AuthorStats will process %d total papers to retrieve citations"
    )
    changeBackend = "Changed backend to %s"
    citationsPaper = "Citations for each paper"
    citationsYear = "Citations per year"
    emptyResponse = "Empty response!"
    errorPlotting = "Something went wrong while plotting..."
    errorReadPage = "Cannot read the page content!"
    meanCitations = "Mean citations"
    noPlot = "Nothing to plot..."
    noPublications = "No publications for this author?"
    paperNumber = "Paper number"
    paperStats = "Stats for paper '%s'"
    paperYear = "Papers per year"
    plotAuthor = "Plotting for author '%s'..."
    plotPaper = "Plotting for paper '%s'..."
    savingCitations = "Saving citation counts..."
    stopReceived = (
        "Received 'stop' signal. "
        + "Interrupting download of information. "
        + "Processing and exiting..."
    )
    totalCitations = "Total citations"


class ParseAccentsStrings:
    """Strings for the physbiblio.parseAccents module"""

    converting = "    -> Converting bad characters in entry %s: "
    infodashes = "         -- "


class PDFStrings(CommonStrings):
    """Strings for the physbiblio.pdf module"""

    copied = "%s copied to %s"
    downloading = "Downloading arXiv PDF from %s"
    e404 = "(404 error on url: %s)"
    errorArxivUrl = "Invalid arXiv PDF url for '%s', probably the field is empty."
    errorCopy = "Impossible to copy %s to %s"
    errorField = "Required field does not exist or is not valid"
    errorFormat = "Invalid format. Using '%.2f'"
    errorGetType = "Impossible to get the type '%s' filename for entry %s"
    errorInvalidSel = (
        "Invalid selection. One among fileType, fileNum or fileName must be given!"
    )
    errorList = "Error in listing the files"
    errorMissingArg = "You should supply a fileType ('doi' or 'arxiv') or a customName!"
    errorRemove = "Impossible to remove file: %s"
    errorSave = "Impossible to save to '%s'"
    errorSize = "Invalid size. It must be a number!"
    errorUnits = "Invalid units. Changing to 'MB'."
    folderMissing = "PDF folder is missing: %s. Creating it."
    invalidCheckFileArg = "Invalid argument to checkFile!"
    listing = "Listing file for entry '%s', located in %s:"
    pdfNotFound = "ArXiv PDF for '%s' not found "
    pdfPresent = "There is already a pdf and overwrite not requested."
    rename = "Renaming %s to %s"
    removed = "File %s removed"
    saved = "File saved to %s"
    spareFound = "Spare PDF folders found: %d\n%s\nThey will be removed now."
    wrongType = "Wrong filename type! %s"


class TablesDefStrings:
    """Strings for the physbiblio.tablesDef module"""

    catsDescs = {
        "idCat": "Unique ID that identifies the category",
        "name": "Name of the category",
        "description": "Description of the category",
        "parentCat": "Parent category",
        "comments": "Comments",
        "ord": "Ordering when plotting (not yet implemented)",
    }
    entriesCatsDescs = {
        "idEnC": "Unique identifier",
        "bibkey": "Corresponding bibtex key",
        "idCat": "Corresponding category ID",
    }
    entriesDescs = {
        "bibkey": "Unique bibtex key that identifies the bibliographic element",
        "inspire": "INSPIRE-HEP ID of the record",
        "arxiv": "arXiv ID of the record",
        "ads": "NASA ADS ID of the record",
        "scholar": "Google Scholar ID of the record",
        "doi": "DOI of the record",
        "isbn": "ISBN code of the record",
        "year": "Year of publication",
        "link": "Web link to article or additional material",
        "comments": "Comments",
        "old_keys": "Previous bibtex keys of the record",
        "crossref": "Bibtex crossref reference",
        "bibtex": "Bibtex entry",
        "firstdate": "Date of first appearance",
        "pubdate": "Date of publication",
        "exp_paper": "(T/F) The entry is a collaboration paper of some experiment",
        "lecture": "(T/F) The entry is a lecture",
        "phd_thesis": "(T/F) The entry is a PhD thesis",
        "review": "(T/F) The entry is a review",
        "proceeding": "(T/F) The entry is a proceeding",
        "book": "(T/F) The entry is a book",
        "noUpdate": "(T/F) The entry has been created/modified by the user "
        + "and/or should not be updated",
        "marks": "Mark the record",
        "abstract": "Abstract of the record",
        "bibdict": "Dictionary with fields of the bibtex entry from bibtexparser",
        "citations": "Most recent number of citations from INSPIRE",
        "citations_no_self": "Most recent number of citations from INSPIRE, excluding self cites",
    }
    entriesExpsDescs = {
        "idEnEx": "Unique identifier",
        "bibkey": "Corresponding bibtex key",
        "idExp": "Corresponding experiment ID",
    }
    expsCatsDescs = {
        "idExC": "Unique identifier",
        "idExp": "Corresponding experiment ID",
        "idCat": "Corresponding category ID",
    }
    expsDescs = {
        "idExp": "Unique ID that identifies the experiment",
        "name": "Name of the experiment",
        "comments": "Description or comments",
        "homepage": "Web link to the experiment homepage",
        "inspire": "INSPIRE-HEP ID of the experiment record",
    }
    searchesDescs = {
        "idS": "Unique identifier",
        "name": "Custom name of the search/replace",
        "count": "Order of the entry in the cronology",
        "searchDict": "Dictionary of fields to be passed to fetchByDict",
        "limit": "The max number of results in the search",
        "offset": "The offset in the search",
        "replaceFields": "List of fields used in replacement",
        "manual": "Manually saved",
        "isReplace": "(T/F) A replacement or a simple search",
    }
    settingsDescs = {
        "id": "Unique identifier",
        "name": "name of the setting",
        "value": "value of the setting",
    }


class ViewStrings(CommonStrings):
    """Strings for the physbiblio.view module"""

    errorLink = "Impossible to get the '%s' link for entry '%s'"
    warningInvalidUse = (
        "Invalid selection or missing information.\n"
        + "Use one of (arxiv|doi|inspire|file) and "
        + "check that all the information is available "
        + "for entry '%s'."
    )



#!/usr/bin/env python
"""Test file for the physbiblio.argParser module.

This file is part of the physbiblio package.
"""
import traceback
import unittest

try:
    import physbiblio
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


class TestPackage(unittest.TestCase):
    """Test package properties"""

    def test_package(self):
        """Test that the physbiblio package has some basic properties"""
        self.assertTrue(hasattr(physbiblio, "__author__"))
        self.assertTrue(hasattr(physbiblio, "__email__"))
        self.assertTrue(hasattr(physbiblio, "__version__"))
        self.assertTrue(hasattr(physbiblio, "__version_date__"))
        self.assertTrue(hasattr(physbiblio, "__all__"))
        self.assertTrue(hasattr(physbiblio, "__recent_changes__"))


if __name__ == "__main__":
    unittest.main()



from .common import CommonStrings
from .main import ErrorsStrings, InspireStatsStrings


class CommonGUIStrings:
    """Some strings used in more than class of this module"""

    attentionE = "Attention!"
    cats = "Categories"
    catsAssociated = "Associated categories: {ca}"
    catsInserted = "Categories for '%s' successfully inserted"
    clickMissingIndex = "Click on missing index"
    config = "Configuration"
    elementFailed = "Failed in inserting entry '%s'\n"
    elementFailedBibtex = "Failed in inserting entry '%s'. Corrupted bibtex?\n"
    elementImported = "Entries successfully imported: %s"
    elementInserted = "Element '%s' successfully inserted.\n"
    entriesCorrespondent = "Corresponding entries: {en}\n"
    entryNotInDb = "The entry '%s' is not in the database!"
    expNotInDb = "The experiment ID %s is not in the database!"
    expsAssociated = "Associated experiments: {ex}"
    featureNYI = "This feature is not implemented"
    filterEntries = "Filter entries"
    nothingChanged = "Nothing changed"
    nothingToDo = "Nothing to do..."
    openEntryList = "Open list of corresponding entries"
    openEntryListNewTab = "Open in new tab"
    openLinkFailed = "Opening link '%s' failed!"
    winTitle = "PhysBiblio"
    winTitleModified = "PhysBiblio*"


class BasicDialogsStrings(CommonStrings):
    """Strings for the physbiblio.gui.basicDialogs module"""

    dir2Use = "Directory to use:"
    fn2Use = "Filename to use:"


class BibWindowsStrings(CommonStrings, CommonGUIStrings):
    """Strings for the physbiblio.gui.bibWindows module"""

    class Acts(CommonStrings):
        """Strings used in the definition of actions"""

        absD = "Abstract of arxiv:%s"
        absF = "No arxiv number for entry '%s'!"
        absS = "Starting the abstract download process, please wait..."
        arxDowD = (
            "Download of PDF for arXiv:%s completed! Please check that it worked..."
        )
        arxDowP = "downloading PDF for arxiv:%s..."
        arxLoad = "Load abstract"
        arxMore = "Get more fields"
        arxTit = "arXiv"
        cantDelOld = "Cannot delete old items!"
        cantInsNew = "Cannot insert new item!"
        catsIns = "Categories successfully inserted"
        catsInsE = "Categories for '%s' successfully inserted"
        citSel = "Citations: %d"
        citSelNS = "without self citations: %d"
        countSel = "Selected elements: %d"
        cpAllPdf = "Copy all the corresponding PDF"
        cpBib = "bibtex(s)"
        cpCite = "\cite{key(s)}"
        cpDir = "Name of the directory for associated files"
        cpKey = "key(s)"
        cpTit = "Copy to clipboard"
        eDesc = "--Entry: %s--"
        emptyMenu = "--Empty selection--"
        emptyKeyBib = "Empty bibtex and/or bibkey!"
        expBib = "Export in a .bib file"
        expIns = "Experiments successfully inserted"
        expInsE = "Experiments for '%s' successfully inserted"
        expW = (
            "Warning: you can just add experiments to the selected entries, not delete!"
        )
        insCit = "Citation statistics"
        insCitCo = "Update citation count"
        insCompl = "Complete info (ID and auxiliary info)"
        insComplTip = (
            "If the record has no associated INSPIRE ID,\n"
            + "look for the record using its bibkey or the DOI/arXiv information.\n"
            + "Once a ID is found, use it to add more information"
        )
        insRel = "Reload bibtex"
        insTit = "INSPIRE-HEP"
        insUpd = "Update bibtex"
        lnArx = "Open into arXiv"
        lnDoi = "Open DOI link"
        lnIns = "Open into INSPIRE-HEP"
        lnTit = "Links"
        maAdd = "Mark as '%s'"
        maAddL = "Mark all as '%s'"
        maInv = "Invalid mark: '%s'"
        maRem = "Unmark as '%s'"
        maRemL = "Unmark all as '%s'"
        maTit = "Marks"
        maUpd = "updateMark: '%s', entries: %s"
        notFoundFor = "KeyError: '%s' not found for '%s'"
        pdfAddF = "Could not copy the new file!"
        pdfAddS = "PDF successfully copied!"
        pdfAddW = "Where is the file located?"
        pdfArxW = "Download arXiv PDF"
        pdfC = "Copy %s PDF"
        pdfCp = "Copy %s"
        pdfD = "Delete %s PDF"
        pdfDel = "Delete %s"
        pdfDelAsk = "Do you really want to delete the %s file for entry %s?"
        pdfDelP = "deleting %s file..."
        pdfDOIA = "Assign DOI PDF"
        pdfGen = "Add generic file"
        pdfM = "Manage %s PDF"
        pdfMg = "Manage %s"
        pdfO = "Open %s PDF"
        pdfOpen = "Open %s"
        pdfOpenDir = "Open directory"
        pdfSaveM = "Where do you want to save the PDF files?"
        pdfSaveS = "Where do you want to save the PDF %s?"
        pdfTit = "Files"
        reloadFail = "Impossible to reload content."
        selCat = "Select categories"
        selExp = "Select experiments"
        totCitSel = "Total citations of the selection: %d"
        totCitSelNS = "without self citations: %d"
        tyAdd = "Set '%s'"
        tyAddL = "Set '%s' for all"
        tyInv = "Invalid type: '%s'"
        tyRem = "Unset '%s'"
        tyRemL = "Unset '%s' for all"
        tyTit = "Type"

    class ED:
        """Strings for the EditBibtexDialog"""

        bibEmpty = "Invalid form contents: empty bibtex!"
        bibKeyErr = "Invalid form contents: bibtex key will be taken from bibtex!"
        bibNotValid = "Invalid form contents: cannot read bibtex properly!"
        notValidTex = "not valid bibtex!"
        title = "Edit bibtex entry"
        type_ = "Entry type"

    class Info:
        """Strings for showing info on a bibtex entry"""

        author = "<b>%s</b>"
        cats = "Categories: <i>%s</i>"
        citIns = "Last citation count obtained from INSPIRE: %s"
        citNoSelf = " (%s excluding self citations)"
        comm = "<br/>Comments:<br/>%s"
        exps = "Experiments: <i>%s</i>"
        keyErr = "KeyError: '%s' not in %s"
        pubErr = "KeyError: 'journal', 'volume', 'year' or 'pages' not in %s"
        useWith = "(use with '<u>\cite{%s}</u>')"

    class LW:
        """Strings used in the BibtexListWindow class"""

        copyToClipT = "--Copy to clipboard--"
        currCountLabel = "Currently shown: %d"
        errEntry = "The entry cannot be found!"
        errIdx = "The index is not valid!"
        errReadTab = "Error in reading table content"
        filterBib = "Filter bibliography"
        lastQuery = "Last query to bibtex database: \t%s\t\t"
        lastQueryA = " - arguments:\t%s"
        openPDF = "opening PDF..."
        pdfc = "PDF"
        selAllD = "Select all the elements"
        selAllT = "&Select all"
        selCleD = "Discard the current selection and hide checkboxes"
        selCleT = "&Clear selection"
        selComD = "Selection of elements completed"
        selComT = "Selection &completed"
        selEnD = "Select entries from the list"
        selEnT = "&Select entries"
        selNonD = "Unselect all the elements"
        selNonT = "&Unselect all"
        selTwo = "(Select exactly two entries to enable merging them)"
        toolBar = "Bibs toolbar"
        typec = "Type"

    class SR:
        """Strings for the search and replace dialog"""

        andAlso = "and also:"
        cats = "Categories"
        deleteRow = "Delete the current row"
        errorProcessSaved = (
            "Something went wrong when processing " + "the saved search/replace:\n%s"
        )
        errorProcessSavedF = (
            "Something went wrong when processing " + "the saved search fields:\n%s"
        )
        exps = "Experiments"
        fromField = "From field:"
        intoField = "Into field:"
        invalidPrev = "Invalid previous! set to empty.\n%s"
        invalidPrevKey = "Invalid previous, some key is missing!\n%s"
        marks = "Marks"
        maxRes = "Max number of results:"
        missingIx = "Missing index %d (there are %d elements)"
        missingLog = "Missing or wrong 'logical' in line %d"
        missingType = "Missing or wrong 'type' in line %d"
        regex = "regex:"
        replace = "Replace:"
        startFrom = "Start from:"
        text = "Text"
        type_ = "Type"

    abstractTitle = "<b>Abstract:</b><br/>"
    addLine = "Add another line"
    addLineDesc = "Click here if you want more fields:"
    bibSaved = "Bibtex entry saved"
    cantInsMod = "Cannot insert/modify the entry!"
    cantUpdateKey = "Cannot update bibtex key: already present. Restoring previous one."
    copyToClipb = "Copying to clipboard: '%s'"
    btmdInvalid = "BibTableModel.data(): invalid %s"
    deleteAsk = "Do you really want to delete this bibtex entry (bibkey = '%s')?"
    deleted = "Bibtex entry deleted"
    emptyKey = "ERROR: empty bibkey!"
    emptyBib = "ERROR: empty bibtex!"
    errorLatexImg = "Error when converting latex to image"
    fieldsFromArx = "Import fields from arXiv"
    keyMiss = "Key missing: '%s'"
    latexDone = "Latex processing done!"
    mergeBibs = "Merge bibtex entries"
    missMarks = "missing key: 'marks' in %s"
    newKey = "New bibtex key (%s) for element '%s'..."
    newTab = "Open results in new tab"
    noModif = "No modifications to bibtex entry"
    noPDF = "no PDF"
    parseBibErr = "Problem in parsing the following bibtex code:\n%s"
    parseLatex = "Parsing LaTeX..."
    pdfOg = "Opening %s PDF..."
    pdfOpen = "Opening %s..."
    processLatex = "%sProcessing LaTeX formulas..."
    runASave = "Run and save"
    searchBibs = "Search bibtex entries"
    updateKey = "Updating bibtex '%s'..."
    whatPDF = "What PDF of this entry (%s) do you want to open?"
    wrongEdit = "Wrong 'edit', it is not an ID: '%s'"


class CatWindowsStrings(CommonStrings, CommonGUIStrings):
    """Strings for the physbiblio.gui.catWindows module"""

    addNew = "Add new category"
    addSub = "Add subcategory"
    askDelete = "Do you really want to delete this category (ID = '%s', name = '%s')?"
    askExp = "Ask experiments"
    catDeleted = "Category deleted"
    catDescr = "--Category: %s--"
    catEdit = "Edit category"
    catId = "{idC}: {cat}\n"
    catSaved = "Category saved"
    emptyName = "ERROR: empty category name"
    failedFind = "Failed in finding category"
    filterCat = "Filter categories"
    invalidCat = "Invalid idCat in previous selection: %s"
    markCatBibK = "Mark categories for the following entry:<br><b>key</b>:<br>%s<br>"
    markCatBibKAT = (
        "Mark categories for the following "
        + "entry:<br><b>key</b>:<br>%s<br>"
        + "<b>author(s)</b>:<br>%s<br>"
        + "<b>title</b>:<br>%s<br>"
    )
    markCatExpI = (
        "Mark categories for the following experiment:<br><b>id</b>:<br>%s<br>"
    )
    markCatExpINC = (
        "Mark categories for the following "
        + "experiment:<br><b>id</b>:<br>%s<br>"
        + "<b>name</b>:<br>%s<br>"
        + "<b>comments</b>:<br>%s<br>"
    )
    noModifications = "No modifications to categories"
    selectCat = "Select the desired category (only the first one will be considered):"
    selectCats = "Select the desired categories:"
    selectParent = "Select parent"
    updateCat = "Updating category %s..."


class CommonClassesStrings(CommonStrings):
    """Strings for the physbiblio.gui.commonClasses module"""

    alreadyExisting = " - already existing"
    dataListNotDef = "dataList is not defined!"
    invalidIdentif = "Invalid identifier in previous selection: %s"
    invalidIndexTMP = "Invalid index '%s' in TreeModel.parent"
    invalidParentTM = "Invalid parent '%s' in TreeModel.%s"
    missElement = "Missing element"
    openFailed = "Opening link for '%s' failed!"
    openSuccess = "Opening link '%s' for entry '%s' successful!"


class DialogWindowsStrings(CommonStrings, CommonGUIStrings):
    """Strings for the physbiblio.gui.dialogWindows module"""

    advImpMethod = "Select method: "
    advImpResults = "Advanced import - results"
    advImpSearch = "Search string: "
    advImpTitle = "Advanced import"
    arxCat = "Select category: "
    arxDailyTitle = "Browse arxiv daily"
    arxResTitle = "ArXiv daily listing - results"
    arxSub = "Subcategory: "
    askCats = "Ask categories at the end?"
    bibName = "Bib file name:"
    clearLogAsk = "Are you sure you want to clear the log file?"
    clearLogDone = "Log file cleared."
    clearLogFailClear = "Impossible to clear log file!"
    clearLogFailRead = "Impossible to read log file!"
    clearLogTitle = "Clear log file"
    colsAvailable = "Available columns"
    colsDragAndDrop = "Drag and drop items to order visible columns"
    colsSelected = "Selected columns"
    dirPDFSave = "Directory for saving PDF files:"
    errorAF = "self.abstractFormulas not present in DailyArxivSelect. Eprint: %s"
    exportAddTex = "Add more tex files"
    exportRemove = "Remove unused bibtexs?"
    exportReorder = "Reorder existing bibtexs? (includes removing unused ones)"
    exportTexFile = "Export for tex file"
    exportUpdate = "Update existing bibtexs?"
    globalSett = "global setting"
    importSelRes = (
        "This is the list of elements found.\nSelect the ones that you want to import:"
    )
    invalidData = "Data not valid"
    invalidLoggingLevel = "Invalid string for 'loggingLevel' param. Reset to default"
    invalidParamkey = "Invalid paramkey: '%s'"
    invalidText = "Invalid text: %s"
    logFileContent = "Log File Content"
    logFileName = "Name for the log file"
    logFileRead = "Reading %s"
    redirectPrint = "Redirect print"
    selFile = "Select file"
    texNames = "Tex file name(s):"
    whereExportBibs = "Where do you want to export the entries?"
    whichTexFiles = "Which is/are the *.tex file(s) you want to compile?"


class ExpWindowsStrings(CommonStrings, CommonGUIStrings):
    """Strings for the physbiblio.gui.expWindows module"""

    addNew = "Add new experiment"
    askDelete = "Do you really want to delete this experiment (ID = '%s', name = '%s')?"
    emptyName = "ERROR: empty experiment name"
    expDeleted = "Experiment deleted"
    expDescr = "--Experiment: %s--"
    expEdit = "Edit experiment"
    expId = "{idE}: {exp}\n"
    expSaved = "Experiment saved"
    failedFind = "Failed in finding experiment"
    filterExp = "Filter experiment"
    listTitle = "List of experiments"
    markExpK = "Mark experiments for the following entry:<br><b>key</b>:<br>%s<br>"
    markExpKAT = (
        "Mark experiments for the following entry:<br>"
        + "<b>key</b>:<br>%s<br>"
        + "<b>author(s)</b>:<br>%s<br>"
        + "<b>title</b>:<br>%s<br>"
    )
    noModifications = "No modifications to experiments"
    selectDesired = "Select the desired experiments:"
    updateExp = "Updating experiment %s..."


class InspireStatsGUIStrings(InspireStatsStrings):
    """Strings for the physbiblio.gui.inspireStatsGUI module"""

    citations = "Citations"
    citationsPaper = "Citations for each paper"
    citationsYear = "Citations per year"
    datefmt = "%d/%m/%Y"
    hIndexE = "ND"
    hIndexV = "Author h index: %s"
    lineMoreInfo = "Click on the line to have more information:"
    linesMoreInfo = "Click on the lines to have more information:"
    meanCitations = "Mean citations"
    paperNumber = "Paper number"
    paperYear = "Papers per year"
    plotSaved = "Plot saved."
    plotsSaved = "Plots saved."
    totalCitations = "Total citations"
    xInDateIsD = "%s in date %s is %d"
    xInDateIsF = "%s in date %s is %.2f"
    xInYearIs = "%s in year %d is: %d"
    yearsForAvgC = "years is:"
    yearsForAvgI = "Invalid number of years (must be integer)"
    yearsForAvgT = "%.2f (%d citations in %d years)"
    yearsForAvgY = "Average citation count in the last"
    whereSavePlot = "Where do you want to save the plot of the stats?"
    whereSavePlots = "Where do you want to save the plots of the stats?"


class MainWindowStrings(CommonStrings, ErrorsStrings, CommonGUIStrings):
    """Strings for the physbiblio.gui.mainWindows module"""

    class Act:
        """Strings used in the definitions of the actions"""

        abD = "Show About box"
        abT = "&About"
        advImpD = "Open the advanced import window"
        advImpT = "&Advanced Import"
        arxBD = "Browse most recent arXiv listings"
        arxBT = "Browse last ar&Xiv listings"
        arxID = "Get info from arXiv"
        arxIT = "Info from ar&Xiv"
        autD = "Search publication and citation stats of an author from INSPIRES"
        autT = "&AuthorStats"
        bibM = "&Bibliography"
        bibND = "New bibliographic item"
        bibNT = "New &Bib item"
        catD = "Manage Categories"
        catM = "&Categories"
        catT = "&Categories"
        catND = "New Category"
        catNT = "Ne&w Category"
        ccD = (
            "Update the citation count of all bibtexs, "
            + "using information from INSPIRE"
        )
        ccT = "Update citation count"
        chaD = "Show recent changes"
        chaT = "&Changelog"
        cleBD = "Clean all the bibtexs"
        cleBT = "&Clean bibtexs"
        cleED = "Remove spare entries from the connection tables."
        cleET = "&Clean spare entries"
        cleFD = "Clean all the bibtexs, starting from a given one"
        cleFT = "C&lean bibtexs (from ...)"
        clePD = "Remove spare PDF folders."
        clePT = "&Clean spare PDF folders"
        corrD = "Find all the bibtexs which contain syntax errors and are not readable"
        corrT = "&Find corrupted bibtexs"
        dbD = "Show some statistics about the current database"
        dbT = "&Database info"
        delete = "Delete '%s'"
        edit = "Edit '%s'"
        editProfD = "Edit profiles"
        editProfT = "&Edit profiles"
        exitD = "Exit application"
        exitT = "E&xit"
        expAD = "Export complete bibliography as *.bib"
        expAT = "Export &all as *.bib"
        expD = "List of Experiments"
        expM = "&Experiments"
        expLD = "Export last query as *.bib"
        expLT = "Ex&port last as *.bib"
        expND = "New Experiment"
        expNT = "&New Experiment"
        expT = "&Experiments"
        expTD = "Export as *.bib the bibliography needed to compile a .tex file"
        expTT = "Export for a *.&tex"
        fileM = "&File"
        findD = "Open the search dialog to filter the bibtex list"
        findT = "&Find Bibtex entries"
        freqRM = "Frequent &replaces"
        freqSM = "Frequent &searches"
        helpM = "&Help"
        impD = "Import the entries from a *.bib file"
        impT = "&Import from *.bib"
        loadInsCatD = (
            "Use INSPIRE-HEP to load and insert bibtex entries, "
            + "then ask the categories for each"
        )
        loadInsCatT = "&Load from INSPIRE-HEP (ask categories)"
        loadInsD = "Use INSPIRE-HEP to load and insert bibtex entries"
        loadInsT = "&Load from INSPIRE-HEP"
        logD = "Show the content of the logfile"
        logT = "Log file"
        manage = "Manage '%s'"
        profD = "Manage profiles"
        profT = "&Profiles"
        refD = "Refresh the current list of entries"
        refT = "&Refresh current entries list"
        rename = "Rename '%s'"
        resD = "Reload the list of bibtex entries"
        resT = "&Reload (reset) main table"
        saveD = "Save the modifications"
        saveT = "&Save database"
        settD = "Save the settings"
        settT = "Settin&gs"
        srD = "Open the search&replace dialog"
        srT = "&Search and replace bibtexs"
        toolbar = "Toolbar"
        toolsM = "&Tools"
        undoD = "Rollback to last saved database state"
        undoT = "&Undo"
        updBD = "Update all the journal info of bibtexs"
        updBT = "&Update bibtexs"
        updPD = (
            "Update all the journal info of bibtexs, "
            + "but with non-standard options (start from, force, ...)"
        )
        updPT = "Update bibtexs (&personalized)"
        updD = "Read a *.bib file and update the existing elements inside it"
        updT = "Update an existing *.&bib file"

    aboutText = (
        "PhysBiblio (<a href='https://github.com/steog88/physBiblio'>"
        + "https://github.com/steog88/physBiblio</a>) is "
        + "a cross-platform tool for managing a LaTeX/BibTeX database. "
        + "It is written in <code>python</code>, "
        + "using <code>sqlite3</code> for the database management "
        + "and <code>PySide</code> for the graphical part."
        + "<br>"
        + "It supports grouping, tagging, import/export, "
        + "automatic update and various different other functions."
        + "<br><br>"
        + "<b>Paths:</b><br>"
        + "<i>Configuration:</i> {cp:}<br>"
        + "<i>Data:</i> {dp:}<br>"
        + "<br>"
        + "<b>Author:</b> Stefano Gariazzo "
        + "<i>&lt;stefano.gariazzo@gmail.com&gt;</i><br>"
        + "<b>Version:</b> {ver:} ({vd:})<br>"
        + "<b>Python version</b>: {pyv:}"
    )
    aboutTitle = "About PhysBiblio"
    arxBrNoCat = "Non-existent category! %s"
    arxIm = "Import from arXiv"
    arxIn = "Get info from arXiv"
    arxInStart = "Starting importing info from arxiv..."
    askADSTokenText = (
        "Before using the ADS service by NASA, you need to obtain an API key:"
        + " the current value is empty.<br/>Sign up for the newest version of ADS search "
        + "at <a href='https://ui.adsabs.harvard.edu'>https://ui.adsabs.harvard.edu</a>,"
        + " visit 'Customize settings', 'API token' and 'Generate a new key'.<br/>"
        + "Then, enter it here and it will be stored in the configuration:"
    )
    askADSTokenTitle = "Token for ADS is missing"
    askOverwriteBib = "Do you want to overwrite the existing .bib file?"
    authorStEmptyName = "Empty name inserted! cannot proceed."
    authorStNameAsk = (
        "Insert the INSPIRE name of the author of which you want "
        + "the publication and citation statistics:"
    )
    authorStStart = "Starting computing author stats from INSPIRE..."
    authorStStats = "Statistics for '%s'"
    authorStNameTit = "Author name?"
    authorStatsTitle = "Author Stats"
    catInserted = "categories for '%s' successfully inserted"
    cheBib = "Check Bibtexs"
    cheBibFound = "%d bad records have been found. Do you want to fix them one by one?"
    cheBibNoInv = "No invalid records found!"
    cheBibStart = "Starting checking bibtexs..."
    cheBibSumNoAct = (
        "These are the bibtex keys corresponding to invalid records:\n%s"
        + "\n\nNo action will be performed."
    )
    citCount = "Citation Count"
    cleBib = "Clean Bibtexs"
    cleBibStart = "Starting cleaning of bibtexs..."
    cleBibStartAsk = (
        "Insert the ordinal number of the bibtex element "
        + "from which you want to start the cleaning:"
    )
    cleBibStartTit = "Where do you want to start cleanBibtexs from?"
    cleanSpareT = "Clean spare entries"
    cleanPDFAsk = (
        "Do you really want to delete the unassociated PDF folders?\n"
        + "There may be some accidental deletion of files, "
        + "especially if you have multiple profiles."
    )
    cleanPDFT = "Clean spare PDF folders"
    configDiscard = "Changes discarded"
    configNewVal = "New value for param %s = %s (old: '%s')"
    configNoChanges = "No changes requested"
    configReload = "Reloading configuration..."
    configSave = "Configuration saved"
    configUseVal = "Using configuration param %s = %s"
    dbLockedAskClose = (
        "The database is locked.\n"
        + "Probably another instance of the program is currently open "
        + "and you cannot save your changes.\n"
        + "For this reason, the current instance may not work properly.\n"
        + "Do you want to close this instance of PhysBiblio?"
    )
    dbStatsText = (
        "The PhysBiblio database currently contains "
        + "the following number of records:\n"
        + "- {bib:} bibtex entries\n"
        + "- {cat:} categories\n"
        + "- {exp:} experiments,\n"
        + "- {catbib:} bibtex entries to categories connections\n"
        + "- {catexp:} experiment to categories connections\n"
        + "- {bibexp:} bibtex entries to experiment connections.\n\n"
        + "The number of currently stored PDF files is {nf:}.\n"
        + "The size of the PDF folder is {pdfs:}."
    )
    dbStatsTitle = "PhysBiblio database statistics"
    emptyFN = "Empty filename given!"
    emptyInFNs = "Empty input filename(s)!"
    emptyOutFN = "Empty output filename!"
    emptyCantProceed = "Empty string! cannot proceed."
    expInserted = "experiments for '%s' successfully inserted"
    exporting = "Exporting..."
    exportAllSaved = "All entries saved into '%s'"
    exportLastDone = "Last fetched entries exported into '%s'"
    exportSelDone = "Current selection exported into '%s'"
    exportWhereG = "Where do you want to export the entries?"
    exportWhereS = "Where do you want to export the selected entries?"
    forceUpdText = (
        "Do you want to force the update of already "
        + "existing items?\n(Only regular articles not "
        + "explicitely excluded will be considered)"
    )
    forceUpdTitle = "Force update:"
    importAllDone = "All entries into '%s' have been imported"
    importFile = "File '%s' imported!"
    importFromWhere = "From where do you want to import?"
    importing = "Importing..."
    importInspAskStr = (
        "Insert the query string you want to use for importing from INSPIRE-HEP:\n"
        + "(It will be interpreted as a list, if possible)"
    )
    importInspire = "Do you want to use INSPIRE to find more information about the imported entries?"
    importInspNoRes = (
        "No results obtained. Maybe there was an error or you interrupted execution."
    )
    importInspStarting = "Starting import from INSPIRE..."
    importInspTit = "Import from INSPIRE-HEP"
    invalidEntry = "Invalid entry"
    invalidIntAsk = (
        "The text you inserted is not an integer. "
        + "I will start from 0.\nDo you want to continue?"
    )
    listSyntax = "Cannot recognize the list sintax. Missing quotes in the string?"
    mainTab = "Main tab"
    newTab = "New tab"
    newTabAsk = "Insert the new tab name"
    newTabName = "New tab name?"
    newVersion = (
        "New version available (%s)!\n"
        + "You can upgrade with `pip install -U physbiblio` "
        + "(with `sudo`, eventually)."
    )
    noNewVersion = "No new versions available!"
    noRes = "No results obtained."
    paperStats = "Paper Stats"
    paperStError = "No results obtained. Maybe there was an error."
    paperStStats = "Statistics for recid:%s"
    queryStr = "Query string?"
    recentCh = "Recent changes"
    recentNew = "New in this <b>version %s</b> (%s):<br>"
    reloadMain = "Reloading main table..."
    replaceAskName = (
        "Insert a name / short description to be able "
        + "to recognise this replace in the future:"
    )
    replaceCompl = (
        "Replace completed.<br><br>"
        + "{suc:} elements successfully processed "
        + "(of which {ncha:} changed), "
        + "{nfai:} failures (see below)."
        + "<br><br><b>Changed</b>: {cha:}"
        + "<br><br><b>Failed</b>: {fai:}"
    )
    replaceEmptyStr = "The string to substitute is empty!"
    replaceEmptyStrAsk = "Empty new string. Are you sure you want to continue?"
    replaceName = "Replace name"
    saveAsk = "Do you really want to save?"
    saveCh = "Changes saved"
    saveNoth = "Nothing saved"
    searchAskDelete = "Are you sure you want to delete the saved search '%s'?"
    searchAskName = (
        "Insert a name / short description to be able "
        + "to recognise this search in the future:"
    )
    searchCantFind = "Cannot find the requested search! id:%s"
    searching = "Searching:\n%s"
    searchName = "Search name"
    srMoreEntries = (
        "Warning: more entries match the current search, showing only the first %d of %d."
        + "\nChange 'Max number of results' in the search form to see more."
    )
    srAskNewName = (
        "Insert a new name / short description "
        + "to be able to recognise this search/replace in the future "
        + "(current name: '%s'):"
    )
    srNewName = "New name"
    trigCat = "categories triggered"
    trigExp = "experiments triggered"
    unsavedChanges = "There may be unsaved changes to the database."
    updateFileD = "File '%s' updated"
    updateFileQ = "File to update?"
    updateInfo = "Update Info"
    updateInfoStart = "Starting generic info update from INSPIRE-HEP..."
    updateStartFrom = "Starting update of bibtexs from %s..."
    updateTitle = "Update Bibtexs"
    updINumText = (
        "Insert the ordinal number of the bibtex element "
        + "from which you want to start the updates:"
    )
    updINumTitle = "Where do you want to start searchOAIUpdates from?"
    wantToExit = "Do you really want to exit?"


class MarksStrings:
    """Strings for the physbiblio.gui.marks module"""

    anys = "Any"
    bad = "Bad"
    favorite = "Favorite"
    important = "Important"
    toBeRead = "To be read"
    unclear = "Unclear"
    marks = "Marks"


class ProfilesManagerStrings(CommonStrings):
    """Strings for the physbiblio.gui.profileManager module"""

    addNew = "Add new?"
    askCancel = (
        "Do you really want to cancel the profile '%s'?\n"
        + "The action cannot be undone!\n"
        + "The corresponding database will not be erased."
    )
    availableProfiles = "Available profiles: "
    cannotCreateEmpty = "Cannot create a new profile if 'name' or 'filename' is empty."
    cannotCreateFieldInUse = "Cannot create new profile: '%s' already in use."
    changingProfile = "Changing profile..."
    completed = "Profile management completed"
    copyFrom = "Copy from:"
    editProfile = "Edit profile"
    errorSwitchLines = "Impossible to switch lines: index out of range"
    missingField = "Missing field: '%s' in %s. Default to empty."
    missingInfo = "Missing info: '%s' in %s. Default to empty."
    missingProfile = "Missing profile: '%s' in %s"
    newCreated = "New profile created."
    noModifications = "No modifications"
    selectProfile = "Select profile"
    splitter = " -- "


class ThreadElementsStrings(CommonStrings, CommonGUIStrings):
    """Strings for the physbiblio.gui.threadElements module"""

    outdatedError = (
        "Error when executing check_outdated. "
        + "Maybe you are using a developing version"
    )
    outdatedWarning = "Error when trying to check new versions. Are you offline?"
    errorsEntries = "Errors for entries:\n%s"



#!/usr/bin/env python
"""Test file for the physbiblio.webimport subpackage.

This file is part of the physbiblio package.
"""
import datetime
import os
import traceback
import unittest

import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

try:
    from physbiblio.config import pbConfig
    from physbiblio.parseAccents import parse_accents_str
    from physbiblio.setuptests import *
    from physbiblio.webimport.webInterf import PBSession, WebInterf, physBiblioWeb
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


@unittest.skipIf(skipTestsSettings.online, "Online tests")
class TestWebImportMethods(unittest.TestCase):
    """Test the functions that import entries from the web.
    Should not fail if everything works fine.
    """

    def test_methods_success(self):
        """Test webimport with known results"""
        self.maxDiff = None
        tests = {
            "arxiv": [
                "1507.08204",
                "@Article{1507.08204,\n         title = "
                + '"{Light sterile neutrinos}",\n          year = "2015",'
                + '\n archiveprefix = "arXiv",\n  primaryclass = '
                + '"hep-ph",\n           doi = '
                + '"10.1088/0954-3899/43/3/033001",\n      abstract = '
                + '"{The theory and phenomenology of light sterile neutrinos'
                + " at the eV mass scale is reviewed. The reactor, Gallium "
                + "and LSND anomalies are briefly described and interpreted "
                + "as indications of the existence of short-baseline "
                + "oscillations which require the existence of light sterile"
                + " neutrinos. The global fits of short-baseline oscillation"
                + " data in 3+1 and 3+2 schemes are discussed, together "
                + "with the implications for beta-decay and neutrinoless "
                + "double-beta decay. The cosmological effects of light "
                + "sterile neutrinos are briefly reviewed and the "
                + "implications of existing cosmological data are discussed."
                + " The review concludes with a summary of future "
                + 'perspectives.}",\n         arxiv = "1507.08204",\n    '
                + '   authors = "S. Gariazzo and C. Giunti and M. Laveder '
                + 'and Y. F. Li and E. M. Zavanin",\n}',
            ],
            "doi": [
                "10.1088/0954-3899/43/3/033001",
                """@article{Gariazzo_2015,
	doi = {10.1088/0954-3899/43/3/033001},
	url = {"""
                + pbConfig.doiUrl
                + """10.1088%2F0954-3899%2F43%2F3%2F033001},
	year = 2015,
	month = {mar},
	publisher = {{IOP} Publishing},
	volume = {43},
	number = {3},
	pages = {033001},
	author = {S Gariazzo and C Giunti and M Laveder and Y F Li and E M Zavanin},
	title = {Light sterile neutrinos},
	journal = {Journal of Physics G: Nuclear and Particle Physics}
}""",
            ],
            "inspire": [
                "Gariazzo:2015rra",
                """@article{Gariazzo:2015rra,
    author = "Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y. F. and Zavanin, E. M.",
    title = "{Light sterile neutrinos}",
    eprint = "1507.08204",
    archivePrefix = "arXiv",
    primaryClass = "hep-ph",
    doi = "10.1088/0954-3899/43/3/033001",
    journal = "J. Phys. G",
    volume = "43",
    pages = "033001",
    year = "2016"
}""",
            ],
        }
        for method, strings in tests.items():
            print(method)
            if method == "arxiv":
                self.assertEqual(
                    physBiblioWeb.webSearch[method]
                    .retrieveUrlFirst(strings[0], searchType="id")
                    .strip(),
                    strings[1].strip(),
                )
                self.assertEqual(
                    physBiblioWeb.webSearch[method]
                    .retrieveUrlAll(strings[0], searchType="id")
                    .strip(),
                    strings[1].strip(),
                )
            else:
                self.assertEqual(
                    physBiblioWeb.webSearch[method]
                    .retrieveUrlFirst(strings[0])
                    .strip(),
                    strings[1].strip(),
                )
                self.assertEqual(
                    physBiblioWeb.webSearch[method].retrieveUrlAll(strings[0]).strip(),
                    strings[1].strip(),
                )

    def test_methods_insuccess(self):
        """Test webimport using missing and/or invalid identifiers"""
        print(physBiblioWeb.webSearch.keys())
        self.maxDiff = None
        tests = {
            "arxiv": ["1801.15000", ""],
            "doi": ["10.1088/9999-3899/43/a/033001", ""],
            "inspire": ["Gariazzo:2014rra", ""],
        }
        self.assertFalse(physBiblioWeb.webSearch["inspire"].retrieveOAIData("9999999"))
        for method, strings in tests.items():
            print(method)
            res = physBiblioWeb.webSearch[method].retrieveUrlFirst(strings[0])
            print(res)
            self.assertEqual(res.strip(), strings[1].strip())
            self.assertEqual(
                physBiblioWeb.webSearch[method].retrieveUrlAll(strings[0]).strip(),
                strings[1].strip(),
            )


class TestWebImportOffline(unittest.TestCase):
    """Offline test for some functions of the webImport package."""

    def test_PBSession(self):
        """test the PBSession class"""
        pbs = PBSession()
        self.assertIsInstance(pbs, requests.Session)
        fr = Retry()
        ha = HTTPAdapter(max_retries=fr)
        with patch(
            "physbiblio.webimport.webInterf.Retry", return_value=fr
        ) as _r, patch(
            "physbiblio.webimport.webInterf.HTTPAdapter", return_value=ha
        ) as _ha, patch(
            "requests.Session.mount"
        ) as _m:
            pbs = PBSession()
            _r.assert_called_once_with(
                total=5,
                backoff_factor=1.0,
                status_forcelist=[429, 500, 502, 503, 504],
                allowed_methods=["HEAD", "GET", "OPTIONS"],
            )
            _ha.assert_called_once_with(max_retries=fr)
            _m.assert_any_call("https://", ha)
            _m.assert_any_call("http://", ha)

    def test_createUrl(self):
        """Test createUrl"""
        pbw = WebInterf()
        pbw.url = pbConfig.inspireLiteratureAPI
        pbw.urlArgs = {"abc": "1", "def": "2"}
        self.assertEqual(
            pbw.createUrl(), "https://inspirehep.net/api/literature/?abc=1&def=2"
        )
        self.assertEqual(
            pbw.createUrl({"abc": "2", "def": "1"}),
            "https://inspirehep.net/api/literature/?abc=2&def=1",
        )
        self.assertEqual(
            pbw.createUrl({}),
            "https://inspirehep.net/api/literature/",
        )
        self.assertEqual(
            pbw.createUrl({}, ""),
            "https://inspirehep.net/api/literature/",
        )
        self.assertEqual(
            pbw.createUrl({"def": "1"}, "abc"),
            "abc?def=1",
        )

    def test_inspire_retrieve(self):
        """Test retrieveUrlFirst and retrieveUrlAll from inspire module"""
        with patch(
            "physbiblio.webimport.inspire.WebSearch.retrieveBibtex",
            autospec=True,
            side_effect=["abc", "def"],
        ) as _f:
            self.assertEqual(
                physBiblioWeb.webSearch["inspire"].retrieveUrlFirst("abc"), "abc"
            )
            _f.assert_called_once_with(
                physBiblioWeb.webSearch["inspire"], "abc", size=1
            )
            _f.reset_mock()
            self.assertEqual(
                physBiblioWeb.webSearch["inspire"].retrieveUrlAll("abc"), "def"
            )
            _f.assert_called_once_with(
                physBiblioWeb.webSearch["inspire"], "abc", size=250
            )

    def test_arxivYear(self):
        """test the arxivDaily method in the arxiv module"""
        self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("abc"), None)
        self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("0123.345"), None)
        self.assertEqual(
            physBiblioWeb.webSearch["arxiv"].getYear("hep-ph/9900000"), "1999"
        )
        self.assertEqual(
            physBiblioWeb.webSearch["arxiv"].getYear("hep-ph/990000"), None
        )
        self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("1234.5678"), "2012")
        self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("1678.56789"), "2016")

    def test_arxivDaily(self):
        """test the arxivDaily method in the arxiv module"""
        with patch("logging.Logger.warning") as _w:
            self.assertFalse(physBiblioWeb.webSearch["arxiv"].arxivDaily("missing"))
            self.assertFalse(
                physBiblioWeb.webSearch["arxiv"].arxivDaily("physics.missing")
            )
        content_example = (
            """<?xml version="1.0" encoding="UTF-8"?>

<rdf:RDF
 xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 xmlns="http://purl.org/rss/1.0/"
 xmlns:content="http://purl.org/rss/1.0/modules/content/"
 xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"
 xmlns:dc="http://purl.org/dc/elements/1.1/"
 xmlns:syn="http://purl.org/rss/1.0/modules/syndication/"
 xmlns:admin="http://webns.net/mvcb/"
>

<channel rdf:about="http://arxiv.org/">
<title>hep-ex updates on arXiv.org</title>
<link>http://arxiv.org/</link>
<description rdf:parseType="Literal">High Energy Physics - """
            + """Experiment (hep-ex) updates on the arXiv.org e-print archive</description>
<dc:language>en-us</dc:language>
<dc:date>2018-05-23T20:30:00-05:00</dc:date>
<dc:publisher>www-admin@arxiv.org</dc:publisher>
<dc:subject>High Energy Physics - Experiment</dc:subject>
<syn:updateBase>1901-01-01T00:00+00:00</syn:updateBase>
<syn:updateFrequency>1</syn:updateFrequency>
<syn:updatePeriod>daily</syn:updatePeriod>
<items>
 <rdf:Seq>
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.08875" />
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.09166" />
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.06430" />
 </rdf:Seq>
</items>
<image rdf:resource="http://arxiv.org/icons/sfx.gif" />
</channel>
<image rdf:about="http://arxiv.org/icons/sfx.gif">
<title>arXiv.org</title>
<url>http://arxiv.org/icons/sfx.gif</url>
<link>http://arxiv.org/</link>
</image>
<item rdf:about="http://arxiv.org/abs/1805.08875">
<title>Measurement of Transverse Single Spin Asymmetries. """
            + """(arXiv:1805.08875v1 [hep-ex])</title>
<link>http://arxiv.org/abs/1805.08875</link>
<description rdf:parseType="Literal">&lt;p&gt;In 2015 the first """
            + """collisions between polarized protons and nuclei occurred at
the Relativistic Heavy Ion Collider (RHIC), at a center-of-mass energy of
$\sqrt{s_{NN}}=200$ GeV. Comparisons between spin asymmetries and
cross-sections in $p+p$ production to those in $p+A$ production provide insight
into nuclear structure, namely nuclear modification factors, nuclear dependence
of spin asymmetries, and comparison to models with saturation effects. The
transverse single-spin asymmetry, $A_{N}$, has been measured in $\pi^{0}$
production in the STAR Forward Meson Spectrometer (FMS), an electromagnetic
calorimeter covering a forward psuedorapidity range """
            + """of $2.6&amp;lt;\eta&amp;lt;4$. Within
this kinematic range, STAR has previously reported the persistence of large
$\pi^0$ asymmetries with unexpected dependences on $p_T$ and event topology in
$p+p$ collisions. This talk will compare these dependences to those in $p+A$
production.
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/hep-ex/1/"""
            + """au:+Dilks_C/0/1/0/all/0/1&quot;&gt;Christopher Dilks&lt;/a&gt; """
            + """(STAR Collaboration)</dc:creator>
</item>
<item rdf:about="http://arxiv.org/abs/1805.09166">
<title>Recent Top quark results from the Tevatron. """
            + """(arXiv:1805.09166v1 [hep-ex] CROSS LISTED)</title>
<link>http://arxiv.org/abs/1805.09166</link>
<description rdf:parseType="Literal">&lt;p&gt;We present recent measurements
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/"""
            + """hep-ex/1/au:+Tuchming_B/0/1/0/all/0/1&quot;&gt;"""
            + """Boris Tuchming&lt;/a&gt;</dc:creator>
</item>
<item rdf:about="http://arxiv.org/abs/1805.06430">
<title>New Perspective on Hybrid Mesons. (arXiv:1805.06430v2 """
            + """[nucl-th] UPDATED)</title>
<link>http://arxiv.org/abs/1805.06430</link>
<description rdf:parseType="Literal">&lt;p&gt;It is thought that """
            + """strong interactions within the Standard Model can generate
bound-states in which non-Abelian gauge-bosons play a dual role, serving both
as force and matter fields.
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/"""
            + """nucl-th/1/au:+Xu_S/0/1/0/all/0/"""
            + """1&quot;&gt;Shu-Sheng Xu&lt;/a&gt;, &lt;a """
            + """href=&quot;http://arxiv.org/find/"""
            + """nucl-th/1/au:+Cui_Z/0/1/0/all/"""
            + """0/1&quot;&gt;Zhu-Fang Cui&lt;/a&gt;, &lt;a """
            + """href=&quot;http://arxiv.org/find/nucl-th/1/au:+"""
            + """Chang_L/0/1/0/all/0/1&quot;&gt;Lei Chang&lt;/"""
            + """a&gt;, &lt;a href=&quot;http://arxiv.org/find/nucl-th/1/au:+"""
            + """Papavassiliou_J/0/1/0/all/0/1&quot;&gt;Joannis """
            + """Papavassiliou&lt;/a&gt;, &lt;a href=&quot;http://arxiv.org/find/"""
            + """nucl-th/1/au:+Roberts_C/0/1/0/all/0/1&quot;"""
            + """&gt;Craig D. Roberts&lt;/a&gt;, &lt;a """
            + """href=&quot;http://arxiv.org/find/nucl-th/"""
            + """1/au:+Zong_H/0/1/0/all/0/1&quot;&gt;Hong-Shi Zong"""
            + """&lt;/a&gt;</dc:creator>
</item>
</rdf:RDF>"""
        )
        with patch(
            "physbiblio.webimport.webInterf.WebInterf.textFromUrl",
            return_value=content_example,
            autospec=True,
        ) as _fromUrl, patch.dict(pbConfig.params, {"maxAuthorNames": 3}, clear=False):
            result = physBiblioWeb.webSearch["arxiv"].arxivDaily("hep-ex")
            _fromUrl.assert_called_once_with(
                physBiblioWeb.webSearch["arxiv"], "https://export.arxiv.org/rss/hep-ex"
            )
            self.assertEqual(
                result[0],
                {
                    "primaryclass": "hep-ex",
                    "author": "Christopher Dilks",
                    "title": "Measurement of Transverse Single Spin Asymmetries.",
                    "abstract": "In 2015 the first collisions between "
                    + "polarized protons and nuclei occurred at the "
                    + "Relativistic Heavy Ion Collider (RHIC), at a "
                    + "center-of-mass energy of $\\sqrt{s_{NN}}=200$ GeV. "
                    + "Comparisons between spin asymmetries and cross-sections "
                    + "in $p+p$ production to those in $p+A$ production "
                    + "provide insight into nuclear structure, namely "
                    + "nuclear modification factors, nuclear dependence "
                    + "of spin asymmetries, and comparison to models "
                    + "with saturation effects. The transverse single-spin "
                    + "asymmetry, $A_{N}$, has been measured in $\\pi^{0}$ "
                    + "production in the STAR Forward Meson Spectrometer "
                    + "(FMS), an electromagnetic calorimeter covering a "
                    + "forward psuedorapidity range of $2.6&lt;\\eta&lt;4$. "
                    + "Within this kinematic range, STAR has previously "
                    + "reported the persistence of large $\\pi^0$ asymmetries "
                    + "with unexpected dependences on $p_T$ and event topology "
                    + "in $p+p$ collisions. This talk will compare "
                    + "these dependences to those in $p+A$ production. ",
                    "cross": False,
                    "version": "1805.08875v1",
                    "eprint": "1805.08875",
                    "authors": ["Christopher Dilks"],
                    "replacement": False,
                },
            )
            self.assertEqual(
                result[1],
                {
                    "primaryclass": "hep-ex",
                    "author": "Boris Tuchming",
                    "title": "Recent Top quark results from the Tevatron.",
                    "abstract": "We present recent measurements ",
                    "cross": True,
                    "version": "1805.09166v1",
                    "eprint": "1805.09166",
                    "authors": ["Boris Tuchming"],
                    "replacement": False,
                },
            )
            self.assertEqual(
                result[2],
                {
                    "primaryclass": "nucl-th",
                    "author": "Shu-Sheng Xu and Zhu-Fang Cui and "
                    + "Lei Chang and others",
                    "title": "New Perspective on Hybrid Mesons.",
                    "abstract": "It is thought that strong interactions "
                    + "within the Standard Model can generate bound-states "
                    + "in which non-Abelian gauge-bosons play a dual role, "
                    + "serving both as force and matter fields. ",
                    "cross": True,
                    "version": "1805.06430v2",
                    "eprint": "1805.06430",
                    "authors": [
                        "Shu-Sheng Xu",
                        "Zhu-Fang Cui",
                        "Lei Chang",
                        "Joannis Papavassiliou",
                        "Craig D. Roberts",
                        "Hong-Shi Zong",
                    ],
                    "replacement": True,
                },
            )


if __name__ == "__main__":
    unittest.main()



"""Module with the classes and functions that manage
the categories windows and panels.

This file is part of the physbiblio package.
"""
import traceback

from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QAction, QCursor
from PySide6.QtWidgets import QLineEdit, QPushButton, QToolTip, QTreeView, QVBoxLayout

try:
    import physbiblio.gui.resourcesPyside6
    from physbiblio.config import pbConfig
    from physbiblio.database import cats_alphabetical, pBDB
    from physbiblio.errors import pBLogger
    from physbiblio.gui.basicDialogs import askYesNo
    from physbiblio.gui.commonClasses import (
        EditObjectWindow,
        LeafFilterProxyModel,
        NamedElement,
        NamedNode,
        PBDialog,
        PBLabel,
        PBMenu,
        TreeModel,
        TreeNode,
    )
    from physbiblio.gui.errorManager import pBGUILogger
    from physbiblio.strings.gui import CatWindowsStrings as cwstr
    from physbiblio.view import pBView
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())


def editCategory(parentObject, mainWinObject, editIdCat=None, useParentCat=None):
    """Open a dialog (`EditCategoryDialog`) to edit a category
    and process the output.

    Parameters:
        parentObject: the parent widget
        mainWinObject: the object which has
            the `statusBarMessage` and `setWindowTitle` methods
        editIdCat: the id of the category to be edited,
            or `None` to create a new category
        useParentCat: the parent category (if any)
            of the one to be edited
    """
    if editIdCat is not None:
        edit = pBDB.cats.getDictByID(editIdCat)
    else:
        edit = None
    newCatWin = EditCategoryDialog(
        parentObject, category=edit, useParentCat=useParentCat
    )
    newCatWin.exec()
    if newCatWin.result:
        data = {}
        for k, v in newCatWin.textValues.items():
            if k == "parentCat":
                try:
                    s = str(newCatWin.selectedCats[0])
                except IndexError:
                    s = "0"
            else:
                s = "%s" % v.text()
            data[k] = s
        if data["name"].strip() != "":
            if "idCat" in data.keys():
                pBLogger.info(cwstr.updateCat % data["idCat"])
                pBDB.cats.update(data, data["idCat"])
            else:
                pBDB.cats.insert(data)
            message = cwstr.catSaved
            mainWinObject.setWindowTitle(cwstr.winTitleModified)
            try:
                parentObject.recreateTable()
            except AttributeError:
                pBLogger.debug(
                    cwstr.noAttribute % ("parentObject", "recreateTable"), exc_info=True
                )
        else:
            message = cwstr.emptyName
    else:
        message = cwstr.noModifications
    try:
        mainWinObject.statusBarMessage(message)
    except AttributeError:
        pBLogger.debug(
            cwstr.noAttribute % ("mainWinObject", "statusBarMessage"), exc_info=True
        )


def deleteCategory(parentObject, mainWinObject, idCat, name):
    """Ask confirmation and eventually delete the selected category

    Parameters:
        parentObject: the parent widget
        mainWinObject: the object which has
            the `statusBarMessage` and `setWindowTitle` methods
        idCat: the id of the category to be deleted
        name: the name of the category to be deleted
    """
    if askYesNo(cwstr.askDelete % (idCat, name)):
        pBDB.cats.delete(int(idCat))
        mainWinObject.setWindowTitle(cwstr.winTitleModified)
        message = cwstr.catDeleted
        try:
            parentObject.recreateTable()
        except AttributeError:
            pBLogger.debug(
                cwstr.noAttribute % ("parentObject", "recreateTable"), exc_info=True
            )
    else:
        message = cwstr.nothingChanged
    try:
        mainWinObject.statusBarMessage(message)
    except AttributeError:
        pBLogger.debug(
            cwstr.noAttribute % ("mainWinObject", "statusBarMessage"), exc_info=True
        )


class CatsModel(TreeModel):
    """Model for the categories tree"""

    def __init__(
        self, cats, rootElements, parent=None, previous=[], multipleRecords=False
    ):
        """Initialize the model, save the data and the initial selection

        Parameters:
            cats: the list of categories records from the database
            rootElements: the list of root elements of the tree
            parent: the parent widget
                (default: None)
            previous: the list of previously selected categories
                (default: an empty list)
            multipleRecords: activate tristate for selecting categories
                for multiple records
                (default False)
        """
        self.cats = cats
        self.rootElements = rootElements
        TreeModel.__init__(self)
        self.parentObj = parent
        self.selectedCats = {}
        self.previousSaved = {}
        for cat in self.cats:
            self.selectedCats[cat["idCat"]] = False
            self.previousSaved[cat["idCat"]] = False
        for prevIx in previous:
            if prevIx in [cat["idCat"] for cat in self.cats]:
                if multipleRecords:
                    self.previousSaved[prevIx] = True
                    self.selectedCats[prevIx] = "p"
                else:
                    self.selectedCats[prevIx] = True
            else:
                pBLogger.warning(cwstr.invalidCat % prevIx)

    def _getRootNodes(self):
        """Obtain the list of named nodes which represent the root
        elements of the tree

        Output:
            a list of `NamedNode`s
        """
        return [
            NamedNode(elem, None, index) for index, elem in enumerate(self.rootElements)
        ]

    def columnCount(self, parent):
        """Return the number of columns, fixed to one"""
        return 1

    def data(self, index, role):
        """Return the cell data for the given index and role

        Parameters:
            index: the `QModelIndex` for which the data are required
            role: the desired Qt role for the data

        Output:
            None if the index or the role are not valid,
            the cell content of properties otherwise
        """
        if not index.isValid():
            return None
        row = index.row()
        column = index.column()
        item = TreeNode.cast(index)
        if item is None:
            return None
        value = item.element.text
        idCat = item.element.idCat
        if (
            role == Qt.CheckStateRole
            and column == 0
            and hasattr(self.parentObj, "askCats")
            and self.parentObj.askCats
        ):
            if self.previousSaved[idCat] == True and self.selectedCats[idCat] == "p":
                return Qt.PartiallyChecked
            elif self.selectedCats[idCat] == False:
                return Qt.Unchecked
            else:
                return Qt.Checked
        if role == Qt.EditRole:
            return value
        if role == Qt.DisplayRole:
            return value
        return None

    def flags(self, index):
        """Return the cell flags for the given index

        Parameter:
            index: the `QModelIndex` for which the data are required

        Output:
            None if the index or the role are not valid,
            the cell flags otherwise
        """
        if not index.isValid():
            return Qt.NoItemFlags
        if index.column() == 0 and (
            hasattr(self.parentObj, "askCats") and self.parentObj.askCats
        ):
            return (
                Qt.ItemIsUserCheckable
                | Qt.ItemIsEditable
                | Qt.ItemIsEnabled
                | Qt.ItemIsSelectable
            )
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def headerData(self, section, orientation, role):
        """Return the header data given section, orientation and role

        Parameters:
            section: the section number
            orientation: horizontal or vertical header
            role: the requested role

        Output:
            the column name ("Name", if the parameters correspond)
            or None
        """
        if orientation == Qt.Horizontal and role == Qt.DisplayRole and section == 0:
            return "Name"
        return None

    def setData(self, index, value, role):
        """Set the cell data for the given index and role

        Parameters:
            index: the `QModelIndex` for which the data are required
            value: the new data value
            role: the desired Qt role for the data

        Output:
            True if correctly completed,
            False if the `index` is not valid
        """
        if not index.isValid():
            return False
        node = TreeNode.cast(index)
        if node is None:
            return False
        element = node.element
        if element is None:
            return False
        idCat = element.idCat
        if role == Qt.CheckStateRole and index.column() == 0:
            self.previousSaved[idCat] = False
            if Qt.CheckState(value) == Qt.Checked:
                self.selectedCats[idCat] = True
            else:
                self.selectedCats[idCat] = False
        self.dataChanged.emit(index, index)
        return True


class CatsTreeWindow(PBDialog):
    """Extension of `PBDialog` that shows the categories tree"""

    def __init__(
        self,
        parent=None,
        askCats=False,
        askForBib=None,
        askForExp=None,
        expButton=True,
        previous=[],
        single=False,
        multipleRecords=False,
    ):
        """Initialize instance parameters and call the function that
        creates the layout.

        Parameters:
            parent (default None): the parent widget
            askCats (default False): if True, enable checkboxes for
                selection of categories
            askForBib (default None): the optional key which identifies
                in the database the bibtex entry for which categories
                are being selected
            askForExp (default None): the optional ID which identifies
                in the database the experiment for which categories
                are being selected
            expButton (default True): if True, add a button to accept
                the widget content and later ask for experiments
            previous (default []): the list of categories that
                must be selected at the beginning
            single (default False): if True, only allow the selection
                of a single category (the parent category, typically).
                Multiple checkboxes can be selected,
                but only the first one will be considered
            multipleRecords: used when dealing with categories
                corresponding to multiple records. Activate a tristate
                checkbox for the initial list of categories, which are
                typically not the same for all the elements in the list
        """
        PBDialog.__init__(self, parent)
        self.setWindowTitle(cwstr.cats)
        self.currLayout = QVBoxLayout(self)
        self.setLayout(self.currLayout)
        self.askCats = askCats
        self.askForBib = askForBib
        self.askForExp = askForExp
        self.expButton = expButton
        self.previous = previous
        self.single = single
        self.multipleRecords = multipleRecords
        self.result = False
        self.marked = []
        self.root_model = None
        self.proxyModel = None
        self.tree = None
        self.menu = None
        self.timer = None
        self.expsButton = None
        self.filterInput = None
        self.newCatButton = None
        self.acceptButton = None
        self.cancelButton = None

        self.setMinimumWidth(400)
        self.setMinimumHeight(600)

        self.createForm()

    def populateAskCat(self):
        """If selection of categories is allowed, add some information
        on the bibtex/experiment for which the categories are requested
        and a simple message, then create few required empty lists
        """
        if self.askCats:
            if self.askForBib is not None:
                try:
                    bibitem = pBDB.bibs.getByBibkey(self.askForBib, saveQuery=False)[0]
                except IndexError:
                    pBGUILogger.warning(
                        cwstr.entryNotInDb % self.askForBib,
                        exc_info=True,
                    )
                    return
                try:
                    if bibitem["inspire"] != "" and bibitem["inspire"] is not None:
                        link = "<a href='%s'>%s</a>" % (
                            pBView.getLink(self.askForBib, "inspire"),
                            self.askForBib,
                        )
                    elif bibitem["arxiv"] != "" and bibitem["arxiv"] is not None:
                        link = "<a href='%s'>%s</a>" % (
                            pBView.getLink(self.askForBib, "arxiv"),
                            self.askForBib,
                        )
                    elif bibitem["doi"] != "" and bibitem["doi"] is not None:
                        link = "<a href='%s'>%s</a>" % (
                            pBView.getLink(self.askForBib, "doi"),
                            self.askForBib,
                        )
                    else:
                        link = self.askForBib
                    bibtext = PBLabel(
                        cwstr.markCatBibKAT
                        % (link, bibitem["author"], bibitem["title"])
                    )
                except KeyError:
                    bibtext = PBLabel(cwstr.markCatBibK % (self.askForBib))
                self.currLayout.addWidget(bibtext)
            elif self.askForExp is not None:
                try:
                    expitem = pBDB.exps.getByID(self.askForExp)[0]
                except IndexError:
                    pBGUILogger.warning(
                        cwstr.expNotInDb % self.askForExp,
                        exc_info=True,
                    )
                    return
                try:
                    exptext = PBLabel(
                        cwstr.markCatExpINC
                        % (self.askForExp, expitem["name"], expitem["comments"])
                    )
                except KeyError:
                    exptext = PBLabel(cwstr.markCatExpI % (self.askForExp))
                self.currLayout.addWidget(exptext)
            else:
                if self.single:
                    comment = PBLabel(cwstr.selectCat)
                else:
                    comment = PBLabel(cwstr.selectCats)
                self.currLayout.addWidget(comment)
            self.marked = []
            self.parent().selectedCats = []
        return True

    def onCancel(self):
        """Reject the dialog content and close the window"""
        self.result = False
        self.close()

    def onOk(self, exps=False):
        """Accept the dialog content
        (update the list of selected categories)
        and close the window.
        May set `self.result` to "Exps"
        for later opening of a new dialog to ask for experiments.

        Parameter:
            exps (default False): if True, set the result to "Exps",
                otherwise to "Ok"
        """
        self.parent().selectedCats = [
            idC
            for idC in self.root_model.selectedCats.keys()
            if self.root_model.selectedCats[idC] == True
        ]
        self.parent().previousUnchanged = [
            idC
            for idC in self.root_model.previousSaved.keys()
            if self.root_model.previousSaved[idC] == True
        ]

        if (
            self.single
            and len(self.parent().selectedCats) > 1
            and self.parent().selectedCats[0] == 0
        ):
            self.parent().selectedCats.pop(0)
            self.parent().selectedCats = [self.parent().selectedCats[0]]
        self.result = "Exps" if exps else "Ok"
        self.close()

    def changeFilter(self, string):
        """When the filter `QLineEdit` is changed,
        update the `LeafFilterProxyModel` regexp filter

        Parameter:
            string: the new filter string
        """
        self.proxyModel.setFilterRegularExpression(str(string))
        self.tree.expandAll()

    def onAskExps(self):
        """Action to perform when the selection of categories
        will be folloed by the selection of experiments.
        Call `self.onOk` with `exps = True`.
        """
        self.onOk(exps=True)

    def onNewCat(self):
        """Action to perform when the creation
        of a new category is requested
        """
        editCategory(self, self.parent())

    def keyPressEvent(self, e):
        """Manage the key press events.
        Do nothing unless `Esc` is pressed:
        in this case close the dialog
        """
        if e.key() == Qt.Key_Escape:
            self.close()
        elif (
            e.key() in (Qt.Key_Return, Qt.Key_Enter)
            and e.modifiers() == Qt.ControlModifier
        ):
            self.onOk()
        elif e.key() == Qt.Key_F and e.modifiers() == Qt.ControlModifier:
            self.filterInput.setFocus()

    def createForm(self):
        """Create the dialog content, connect the model to the view
        and eventually add the buttons at the end
        """
        self.populateAskCat()

        catsTree = pBDB.cats.getHier()

        self.filterInput = QLineEdit("", self)
        self.filterInput.setPlaceholderText(cwstr.filterCat)
        self.filterInput.textChanged.connect(self.changeFilter)
        self.currLayout.addWidget(self.filterInput)
        self.filterInput.setFocus()

        self.tree = QTreeView(self)
        self.currLayout.addWidget(self.tree)
        self.tree.setMouseTracking(True)
        self.tree.entered.connect(self.handleItemEntered)
        self.tree.doubleClicked.connect(self.cellDoubleClick)
        self.tree.setExpandsOnDoubleClick(False)

        catsNamedTree = self._populateTree(catsTree[0], 0)

        self.root_model = CatsModel(
            pBDB.cats.getAll(),
            [catsNamedTree],
            self,
            self.previous,
            multipleRecords=self.multipleRecords,
        )
        self.proxyModel = LeafFilterProxyModel(self)
        self.proxyModel.setSourceModel(self.root_model)
        self.proxyModel.setFilterCaseSensitivity(Qt.CaseInsensitive)
        self.proxyModel.setSortCaseSensitivity(Qt.CaseInsensitive)
        self.proxyModel.setFilterKeyColumn(-1)
        self.tree.setModel(self.proxyModel)

        self.tree.expandAll()

        self.tree.setHeaderHidden(True)
        # self.tree.doubleClicked.connect(self.askAndPerformAction)

        self.newCatButton = QPushButton(cwstr.addNew, self)
        self.newCatButton.clicked.connect(self.onNewCat)
        self.currLayout.addWidget(self.newCatButton)

        if self.askCats:
            self.acceptButton = QPushButton(cwstr.ok, self)
            self.acceptButton.clicked.connect(self.onOk)
            self.currLayout.addWidget(self.acceptButton)

            if self.expButton:
                self.expsButton = QPushButton(cwstr.askExp, self)
                self.expsButton.clicked.connect(self.onAskExps)
                self.currLayout.addWidget(self.expsButton)

            # cancel button
            self.cancelButton = QPushButton(cwstr.cancel, self)
            self.cancelButton.clicked.connect(self.onCancel)
            self.cancelButton.setAutoDefault(True)
            self.currLayout.addWidget(self.cancelButton)

    def _populateTree(self, children, idCat):
        """Read the list of categories recursively and
        populate the categories tree

        Parameters:
            children: the list of children categories
                of the currently considered one
            idCat: the id of the current category
        """
        name = pBDB.cats.getByID(idCat)[0]["name"]
        children_list = []
        for child in cats_alphabetical(children, pBDB):
            child_item = self._populateTree(children[child], child)
            children_list.append(child_item)
        return NamedElement(idCat, name, children_list)

    def handleItemEntered(self, index):
        """Process event when mouse enters an item and
        create a `QTooltip` which describes the category, with a timer

        Parameter:
            index: a `QModelIndex` instance
        """
        if index.isValid():
            row = index.row()
        else:
            return
        try:
            idString = self.proxyModel.sibling(row, 0, index).data()
        except AttributeError:
            pBLogger.debug("", exc_info=True)
            return
        try:
            idCat, catName = idString.split(": ")
        except AttributeError:
            pBLogger.debug("", exc_info=True)
            return
        idCat = idCat.strip()
        try:
            self.timer.stop()
            QToolTip.showText(QCursor.pos(), "", self.tree.viewport())
        except AttributeError:
            pass
        try:
            catData = pBDB.cats.getByID(idCat)[0]
        except IndexError:
            pBGUILogger.exception(cwstr.failedFind)
            return
        self.timer = QTimer(self)
        self.timer.setSingleShot(True)
        self.timer.timeout.connect(
            lambda: QToolTip.showText(
                QCursor.pos(),
                cwstr.catId.format(idC=idCat, cat=catData["name"])
                + cwstr.entriesCorrespondent.format(en=pBDB.catBib.countByCat(idCat))
                + cwstr.expsAssociated.format(ex=pBDB.catExp.countByCat(idCat)),
                self.tree.viewport(),
                self.tree.visualRect(index),
                3000,
            )
        )
        self.timer.start(500)

    def contextMenuEvent(self, event):
        """Create a right click menu with few actions
        on the selected category

        Parameter:
            event: a `QEvent`
        """
        indexes = self.tree.selectedIndexes()
        try:
            index = indexes[0]
        except IndexError:
            pBLogger.debug(cwstr.clickMissingIndex)
            return
        if index.isValid():
            row = index.row()
        else:
            return
        try:
            idString = self.proxyModel.sibling(row, 0, index).data()
        except AttributeError:
            pBLogger.debug("", exc_info=True)
            return
        try:
            idCat, catName = idString.split(": ")
        except AttributeError:
            pBLogger.debug("", exc_info=True)
            return
        idCat = idCat.strip()

        menu = PBMenu()
        self.menu = menu
        titAction = QAction(cwstr.catDescr % catName)
        titAction.setDisabled(True)
        bibAction = QAction(cwstr.openEntryList)
        bntAction = QAction(cwstr.openEntryListNewTab)
        modAction = QAction(cwstr.modify)
        delAction = QAction(cwstr.delete)
        subAction = QAction(cwstr.addSub)
        menu.possibleActions = [
            titAction,
            None,
            bibAction,
            bntAction,
            None,
            modAction,
            delAction,
            None,
            subAction,
        ]
        menu.fillMenu()

        action = menu.exec(event.globalPos())

        if action == bibAction:
            self.parent().reloadMainContent(pBDB.bibs.getByCat(idCat))
        elif action == bntAction:
            self.parent().reloadMainContent(pBDB.bibs.getByCat(idCat), newTab=catName)
        elif action == modAction:
            editCategory(self, self.parent(), idCat)
        elif action == delAction:
            deleteCategory(self, self.parent(), idCat, catName)
        elif action == subAction:
            editCategory(self, self.parent(), useParentCat=idCat)

    def cellDoubleClick(self, index):
        """Process event when mouse double clicks an item.
        Opens a link if some columns

        Parameter:
            index: a `QModelIndex` instance
        """
        if index.isValid():
            row = index.row()
            col = index.column()
        else:
            return
        try:
            idString = self.proxyModel.sibling(row, 0, index).data()
        except AttributeError:
            pBLogger.debug("", exc_info=True)
            return
        try:
            idCat, catName = idString.split(": ")
        except AttributeError:
            pBLogger.debug("", exc_info=True)
            return
        idCat = idCat.strip()
        self.parent().reloadMainContent(pBDB.bibs.getByCat(idCat))

    def recreateTable(self):
        """Delete the previous widgets and recreate them with new data"""
        self.cleanLayout()
        self.createForm()


class EditCategoryDialog(EditObjectWindow):
    """Create a window for editing or creating a category"""

    def __init__(self, parent=None, category=None, useParentCat=None):
        """Set some properties and create the form content

        Parameters:
            parent: the parent widget
            category: a category record from the database
            useParentCat: the id of the parent category
                of the given one in the tree
        """
        self.acceptButton = None
        self.cancelButton = None
        self.textValues = None
        super(EditCategoryDialog, self).__init__(parent)
        if category is None:
            self.data = {}
            for k in pBDB.tableCols["categories"]:
                self.data[k] = ""
        else:
            self.data = category
        if useParentCat is not None:
            self.data["parentCat"] = useParentCat
            self.selectedCats = [useParentCat]
        elif category is not None:
            try:
                self.selectedCats = [pBDB.cats.getParent(category["idCat"])[0][0]]
            except IndexError:
                self.selectedCats = [0]
        else:
            self.selectedCats = [0]
        self.createForm()

    def onAskParent(self):
        """Open a `CatsTreeWindow` and process its output"""
        selectCats = CatsTreeWindow(
            parent=self,
            askCats=True,
            expButton=False,
            single=True,
            previous=self.selectedCats,
        )
        selectCats.exec()
        if selectCats.result == "Ok":
            try:
                val = self.selectedCats[0]
                self.textValues["parentCat"].setText(
                    "%s - %s" % (str(val), pBDB.cats.getByID(val)[0]["name"])
                )
            except IndexError:
                self.textValues["parentCat"].setText(cwstr.selectParent)

    def createForm(self):
        """Prepare the window widgets"""
        self.setWindowTitle(cwstr.catEdit)

        i = 0
        for k in pBDB.tableCols["categories"]:
            val = self.data[k]
            if k != "idCat" or (k == "idCat" and self.data[k] != ""):
                i += 1
                self.currGrid.addWidget(
                    PBLabel(pBDB.descriptions["categories"][k]), i * 2 - 1, 0
                )
                self.currGrid.addWidget(PBLabel("(%s)" % k), i * 2 - 1, 1)
                if k == "parentCat":
                    try:
                        val = self.selectedCats[0]
                        self.textValues[k] = QPushButton(
                            "%s - %s" % (str(val), pBDB.cats.getByID(val)[0]["name"]),
                            self,
                        )
                    except IndexError:
                        self.textValues[k] = QPushButton(cwstr.selectParent, self)
                    self.textValues[k].clicked.connect(self.onAskParent)
                else:
                    self.textValues[k] = QLineEdit(str(val))
                if k == "idCat":
                    self.textValues[k].setEnabled(False)
                self.currGrid.addWidget(self.textValues[k], i * 2, 0, 1, 2)

        # OK button
        self.acceptButton = QPushButton(cwstr.ok, self)
        self.acceptButton.clicked.connect(self.onOk)
        self.currGrid.addWidget(self.acceptButton, i * 2 + 1, 1)

        # cancel button
        self.cancelButton = QPushButton(cwstr.cancel, self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        self.currGrid.addWidget(self.cancelButton, i * 2 + 1, 0)

        self.setGeometry(100, 100, 400, 50 * i)
        self.centerWindow()
